"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.WOQLEditor = void 0;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _Editor = require("./Editor");

var _LanguageSwitcher = require("./LanguageSwitcher");

var _constants = require("./constants.querypane");

var _queryPaneUtils = require("./queryPaneUtils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

/**
 * Controls the display of query viewer and editor
 */
var WOQLEditor = function WOQLEditor(_ref) {
  var language = _ref.language,
      content = _ref.content,
      editable = _ref.editable,
      setEditorContent = _ref.setEditorContent,
      setMainError = _ref.setMainError,
      setWOQLQuery = _ref.setWOQLQuery;
  WOQLEditor.propTypes = {
    language: _propTypes["default"].string,
    content: _propTypes["default"].string,
    setMainError: _propTypes["default"].func,
    setWOQLQuery: _propTypes["default"].func,
    setEditorContent: _propTypes["default"].func.isRequired,
    editable: _propTypes["default"].bool
  };
  WOQLEditor.defaultProps = {
    content: '',
    language: "js",
    editable: true
  };

  function onBlur(value) {
    /*
    *save the editor string
    */
    setEditorContent(value);
    var woql = checkContent();

    if (woql) {
      if (setWOQLQuery) setWOQLQuery(woql);
    }
  }
  /*
  * onBlur
  */


  function checkContent() {
    //sets errors internally if doesn't work
    setMainError(false);

    if (content) {
      try {
        var woql = (0, _queryPaneUtils.makeWOQLFromString)(content, language);
        /*
        * check if the query is an update query and need a commitSMS
        */
        // setContainsUpdate(woql.containsUpdate())

        return woql;
      } catch (err) {
        console.log(err);
        setMainError(err);
        return false;
      }
    }

    return false;
  }

  return /*#__PURE__*/_react["default"].createElement(_react["default"].Fragment, null, editable && /*#__PURE__*/_react["default"].createElement(_Editor.CodeEditor, {
    onBlur: onBlur,
    text: content,
    language: language
  }), !editable && /*#__PURE__*/_react["default"].createElement(_Editor.CodeViewer, {
    text: content,
    language: language
  }));
};

exports.WOQLEditor = WOQLEditor;