"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Timeline = void 0;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _EventsBar = require("./EventsBar");

var _helpers = require("../helpers");

var _Constants = _interopRequireDefault(require("../Constants"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var Timeline = function Timeline(props) {
  if (!props.containerWidth) {
    return false;
  }

  var dates = props.values || [];
  if (dates.length === 0) return /*#__PURE__*/_react["default"].createElement("div", null, "NO Value"); // Calculate the distances for all events

  var events = (0, _helpers.cummulativeSeparation)(dates, props.labelWidth, props.minEventPadding, props.maxEventPadding, props.linePadding);
  var visibleWidth = props.containerWidth - 80;
  var totalWidth = Math.max(events[events.length - 1].distance + props.linePadding, visibleWidth);
  var barPaddingRight = 0;
  var barPaddingLeft = 0;

  if (!props.isOpenEnding) {
    barPaddingRight = totalWidth - events[events.length - 1].distance;
  }

  if (!props.isOpenBeginning) {
    barPaddingLeft = events[0].distance;
  }

  return /*#__PURE__*/_react["default"].createElement(_EventsBar.EventsBar, {
    gotoPosition: props.gotoPosition,
    width: props.containerWidth,
    height: props.containerHeight,
    events: events,
    isTouchEnabled: props.isTouchEnabled,
    loadNextPage: props.loadNextPage,
    loadPreviousPage: props.loadPreviousPage,
    totalWidth: totalWidth,
    visibleWidth: visibleWidth,
    index: props.index,
    styles: props.styles,
    indexClick: props.indexClick,
    labelWidth: props.labelWidth,
    fillingMotion: props.fillingMotion,
    barPaddingRight: barPaddingRight,
    barPaddingLeft: barPaddingLeft
  }); //};
};
/**
 * The expected properties from the parent
 * @type {Object}
 */


exports.Timeline = Timeline;
Timeline.propTypes = {
  // --- EVENTS ---
  // Selected index
  index: _propTypes["default"].number,
  // Array containing the sorted date strings
  values: _propTypes["default"].arrayOf(_propTypes["default"].shape({
    label: _propTypes["default"].string.isRequired,
    datetime: _propTypes["default"].number.isRequired
  })).isRequired,
  //values: PropTypes.arrayOf(PropTypes.string).isRequired,
  // Function that takes the index of the array as argument
  indexClick: _propTypes["default"].func,
  // --- POSITIONING ---
  // the minimum padding between events
  minEventPadding: _propTypes["default"].number,
  // The maximum padding between events
  maxEventPadding: _propTypes["default"].number,
  // Padding at the front and back of the line
  linePadding: _propTypes["default"].number,
  // The width of the label
  labelWidth: _propTypes["default"].number,
  // --- STYLING ---
  styles: _propTypes["default"].object,
  fillingMotion: _propTypes["default"].object,
  slidingMotion: _propTypes["default"].object,
  isOpenEnding: _propTypes["default"].bool,
  isOpenBeginning: _propTypes["default"].bool,
  // --- INTERACTION ---
  isTouchEnabled: _propTypes["default"].bool //isKeyboardEnabled: PropTypes.bool,

};
/**
 * The values that the properties will take if they are not provided
 * by the user.
 * @type {Object}
 */

Timeline.defaultProps = {
  // --- POSITIONING ---
  minEventPadding: _Constants["default"].MIN_EVENT_PADDING,
  maxEventPadding: _Constants["default"].MAX_EVENT_PADDING,
  linePadding: _Constants["default"].TIMELINE_PADDING,
  labelWidth: _Constants["default"].DATE_WIDTH,
  // --- STYLING ---
  styles: {
    outline: '#dfdfdf',
    background: '#f8f8f8',
    foreground: '#7b9d6f'
  },
  fillingMotion: {
    stiffness: 150,
    damping: 25
  },
  slidingMotion: {
    stiffness: 150,
    damping: 25
  },
  isOpenEnding: true,
  isOpenBeginning: true,
  // --- INTERACTION ---
  isTouchEnabled: true
};