"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.TimelineDot = void 0;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

/**
 * The markup for the dot on the timeline
 */
var TimelineDot = function TimelineDot(props) {
  var dotType = '';

  if (props.index < props.selected) {
    dotType = 'history__nav__dot--back';
  } else if (props.index === props.selected) {
    dotType = 'history__nav__dot--current';
  }

  return /*#__PURE__*/_react["default"].createElement("li", {
    key: props.datetime,
    id: "timeline-dot-".concat(props.datetime),
    className: "history__nav__dot__label",
    onClick: function onClick() {
      return props.onClick(props.index);
    },
    style: {
      left: props.distanceFromOrigin - props.labelWidth / 2
    }
  }, props.label, /*#__PURE__*/_react["default"].createElement("span", {
    key: "dot-dot",
    className: "history__nav__dot ".concat(dotType),
    style: {
      left: props.labelWidth / 2 - props.dotsWidth / 2
    }
  }));
};
/**
 * propTypes
 * @type {Object}
 */


exports.TimelineDot = TimelineDot;
TimelineDot.defaultProps = {
  dotsWidth: 12
};
TimelineDot.propTypes = {
  // The index of the currently selected dot 
  selected: _propTypes["default"].number.isRequired,
  // The index of the present event (used for deciding the styles alongside selected)
  index: _propTypes["default"].number.isRequired,
  // The actual date of the event (used as key and id)
  datetime: _propTypes["default"].number.isRequired,
  // The onClick handler ( in this case to trigger the fillingMotion of the timeline )
  onClick: _propTypes["default"].func.isRequired,
  // The date of the event (required to display it)
  label: _propTypes["default"].string.isRequired,
  // The width you want the labels to be
  labelWidth: _propTypes["default"].number.isRequired,
  // The width of the dots
  dotsWidth: _propTypes["default"].number,
  // The numerical value in pixels of the distance from the origin
  distanceFromOrigin: _propTypes["default"].number.isRequired
};