"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.availableParentsList = exports.addElementToPropertyList = exports.formatDataForTreeChart = exports.formatData = exports.addObjectPropertyRangeItem = exports.formatProperties = exports.addRestictionToProps = void 0;

var _d3Hierarchy = require("d3-hierarchy");

var _elementsName = require("./utils/elementsName");

var _detailsLabels = require("../constants/details-labels");

var _modelTreeUtils = require("./utils/modelTreeUtils");

var _terminusdbClient = require("@terminusdb/terminusdb-client");

function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it["return"] != null) it["return"](); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

var addTypeRange = function addTypeRange(item, newProperty, _rootIndexObj) {
  var range = item["Property Range"];
  /*
  * if the range is a node element
  * I have 2 types of special property LinkProperty and ChoiceProperty
  	*/

  if (_rootIndexObj[range]) {
    newProperty['range'] = range;

    if (_rootIndexObj[range] && _rootIndexObj[range].type === _elementsName.CLASS_TYPE_NAME.CHOICE_CLASS) {
      newProperty['type'] = _elementsName.PROPERTY_TYPE_NAME.CHOICE_PROPERTY;
    } else {
      newProperty['type'] = _elementsName.PROPERTY_TYPE_NAME.OBJECT_PROPERTY;
    }

    return;
  } //"http://terminusdb.com/schema/xdd#url"


  var rangeArr = range.split('#');
  var rangeStr = rangeArr[1];

  if (_detailsLabels.PROPERTY_STRING_BY_RANGE[rangeStr]) {
    newProperty['range'] = _detailsLabels.PROPERTY_STRING_BY_RANGE[rangeStr];
    newProperty['type'] = _elementsName.PROPERTY_TYPE_NAME.STRING_PROPERTY;
    return;
  }

  if (_detailsLabels.PROPERTY_NUMBER_BY_RANGE[rangeStr]) {
    newProperty['range'] = _detailsLabels.PROPERTY_NUMBER_BY_RANGE[rangeStr];
    newProperty['type'] = _elementsName.PROPERTY_TYPE_NAME.NUMERIC_PROPERTY;
    return;
  }

  if (_detailsLabels.PROPERTY_GEO_BY_RANGE[rangeStr]) {
    newProperty['range'] = _detailsLabels.PROPERTY_GEO_BY_RANGE[rangeStr];
    newProperty['type'] = _elementsName.PROPERTY_TYPE_NAME.GEO_PROPERTY;
    return;
  }

  if (_detailsLabels.PROPERTY_TEMPORAL_BY_RANGE[rangeStr]) {
    newProperty['range'] = _detailsLabels.PROPERTY_TEMPORAL_BY_RANGE[rangeStr];
    newProperty['type'] = _elementsName.PROPERTY_TYPE_NAME.TEMPORAL_PROPERTY;
    return;
  }
};
/*
* add restrictions to properties
*/


var addRestictionToProps = function addRestictionToProps(propertiesList, restrDataProvider) {
  var bindings = restrDataProvider.bindings || [];
  bindings.forEach(function (item) {
    var property = propertiesList.get(item['Property']);

    if (property) {
      if (item.cardinality !== "system:unknown") {
        property['cardinality'] = item.cardinality['@value'];
        property['cardinality_start'] = item.cardinality['@value']; //min and max 

        property['min'] = item.cardinality['@value'];
        property['max'] = item.cardinality['@value'];
      }

      if (item.min !== "system:unknown") {
        property['min_start'] = item.min['@value'];
        property['min'] = item.min['@value'];
      }

      if (item.max !== "system:unknown") {
        property['min_stop'] = item.min['@value'];
        property['max'] = item.max['@value'];
      }
    }
  });
};

exports.addRestictionToProps = addRestictionToProps;

var formatProperties = function formatProperties(dataProvider, restrDataProvider, _rootIndexObj) {
  var propertyByDomain = {};
  var objectPropertyRange = {};
  var propertiesList = new Map();
  var bindings = dataProvider.bindings || [];
  bindings.forEach(function (item) {
    var classDomain = item['Property Domain'];

    if (!propertyByDomain[classDomain]) {
      propertyByDomain[classDomain] = [];
    }

    var newProperty = {
      name: item['Property ID'],
      id: getId(item['Property ID']),
      label: item['Property Name']['@value'],
      comment: item['Property Description']['@value'],
      newElement: false,
      domain: classDomain
    };
    addTypeRange(item, newProperty, _rootIndexObj);

    if (newProperty.type === _elementsName.PROPERTY_TYPE_NAME.OBJECT_PROPERTY || newProperty.type === _elementsName.PROPERTY_TYPE_NAME.CHOICE_PROPERTY) {
      // newProperty.range is the className
      if (newProperty.domain && _rootIndexObj[newProperty.domain]) {
        var _classDomain = _rootIndexObj[newProperty.domain];
        addObjectPropertyRangeItem(objectPropertyRange, newProperty, _classDomain);
      } else if (newProperty.domain && !_rootIndexObj[newProperty.domain]) {
        alert(newProperty.domain);
      }
    }

    propertiesList.set(newProperty.name, newProperty);
    propertyByDomain[classDomain].push(newProperty);
  });
  /*
  * add cardinality restrictions
  */

  addRestictionToProps(propertiesList, restrDataProvider);
  return [propertyByDomain, objectPropertyRange, propertiesList];
};
/*
* to be review of don't get the label change
*/


exports.formatProperties = formatProperties;

var addObjectPropertyRangeItem = function addObjectPropertyRangeItem(objectPropertyRange, propertyElement, classDomain) {
  var previewRange = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : undefined;

  /*
  * remove the relation
  */
  if (previewRange) {
    var arrList = objectPropertyRange[previewRange];

    if (arrList) {
      if (arrList.length === 1) objectPropertyRange[previewRange] = [];
      var index = arrList.findIndex(function (item) {
        return item.name === propertyElement.name;
      });
      arrList.splice(index, 1);
    } //const index= arrList.find(propertyElement.name);

  }

  if (!objectPropertyRange[propertyElement.range]) {
    objectPropertyRange[propertyElement.range] = [];
  }

  objectPropertyRange[propertyElement.range].push(propertyElement);
};

exports.addObjectPropertyRangeItem = addObjectPropertyRangeItem;

var formatData = function formatData(dataProvider, dbName) {
  var _rootIndexObj = (0, _elementsName.getRootIndexObj)(dbName);

  _rootIndexObj.ROOT.children.push(_rootIndexObj[_elementsName.CLASS_TYPE_NAME.CHOICE_CLASSES]);

  _rootIndexObj.ROOT.children.push(_rootIndexObj[_elementsName.CLASS_TYPE_NAME.OBJECT_CLASSES]);

  _rootIndexObj.ROOT.children.push(_rootIndexObj[_elementsName.CLASS_TYPE_NAME.DOCUMENT_CLASSES]);

  addElements(_rootIndexObj, dataProvider);
  return _rootIndexObj;
};

exports.formatData = formatData;

var formatDataForTreeChart = function formatDataForTreeChart(rootElement) {
  var treeModel = (0, _d3Hierarchy.tree)();
  /*
  * When you set a size for the tree,Vtree$$1.size(size); 
  * you are setting a fixed size so that the tree has to conform to that width and height.  
  * When you set a nodeSize, the tree has to be dynamic so it resets the size of the tree.
  */
  // treeModel.nodeSize([200,200]);

  treeModel.nodeSize([200, 200]); //treeModel.size([5000, 1000])   

  var data = (0, _d3Hierarchy.hierarchy)(rootElement);
  var d3Data = treeModel(data);
  var treeNode = d3Data.descendants();
  var descendantsNode = new Map();
  /*
  * list of all object class id
  */

  var objectTypeList = [];
  /*
  *list of all Document class id
  */

  var documentTypeList = [];
  var objectPropertyList = [];
  var objectChoiceList = []; //this._descendantsNode=new Map();

  var _iterator = _createForOfIteratorHelper(treeNode),
      _step;

  try {
    for (_iterator.s(); !(_step = _iterator.n()).done;) {
      var node = _step.value;

      /*if(node.data.type==='Group' && node.x && Math.abs(node.x)<200){
      		node.x=node.x*2;
      		//node.y=node.y/1.5;
      }*/
      if (!descendantsNode.has(node.data.name)) {
        if (node.data.type === _elementsName.CLASS_TYPE_NAME.DOCUMENT_CLASS) {
          documentTypeList.push(node.data.name);
          objectPropertyList.push({
            type: node.data.type,
            value: node.data.name,
            name: node.data.name,
            label: node.data.label
          });
        } else if (node.data.type === _elementsName.CLASS_TYPE_NAME.OBJECT_CLASS) {
          objectTypeList.push(node.data.name);
          objectPropertyList.push({
            type: node.data.type,
            value: node.data.name,
            name: node.data.name,
            label: node.data.label
          });
        } else if (node.data.type === _elementsName.CLASS_TYPE_NAME.CHOICE_CLASS) {
          objectChoiceList.push({
            value: node.data.name,
            name: node.data.name,
            label: node.data.label
          });
        }
      }

      descendantsNode.set(node.data.name, node);
    }
  } catch (err) {
    _iterator.e(err);
  } finally {
    _iterator.f();
  }

  return [descendantsNode, objectTypeList, documentTypeList, objectPropertyList, objectChoiceList];
};

exports.formatDataForTreeChart = formatDataForTreeChart;

var addElementToPropertyList = function addElementToPropertyList(elementsObj, objectPropertyList) {
  objectPropertyList.push({
    type: elementsObj.type,
    value: elementsObj.name,
    name: elementsObj.name,
    label: elementsObj.label
  });
};

exports.addElementToPropertyList = addElementToPropertyList;
var SCOPED_VALUE_ID = "terminusdb:///schema#ScopedValue";
var BOX_ID = "terminusdb:///schema#Box";

var getLabel = function getLabel(item, idName) {
  if (item['Class Name']['@value']) {
    return item['Class Name']['@value'];
  }

  if (idName.indexOf(":")) {
    var idArr = idName.split(":");
    return idArr[1];
  }

  return idName;
}; //terminusdb:///schema#Box


var getId = function getId(classId) {
  return _terminusdbClient.UTILS.shorten(classId); //classId.replace("terminusdb:///schema#","");
};
/*
* abstract value 'Yes/No'
*/


var getAbstractValue = function getAbstractValue(item) {
  return item['Abstract']['@value'] === 'Yes' ? true : false;
};

var getClassType = function getClassType(classId, item, _rootIndexObj) {
  /*
  * check if it is an enumClass, 
  * Enum Type is an special class No properties no children but can have a Object Type
  * as parent
  */
  if (isEnumClassType(_rootIndexObj[classId], item, _rootIndexObj)) return;
  /*
  * I have to find the object type
  */

  if (Array.isArray(item['Parents'])) {
    item['Parents'].forEach(function (parentId) {
      /*
      * if I know the type and it is Document, the type can not change.
      */
      if (_rootIndexObj[classId]['type'] === _elementsName.CLASS_TYPE_NAME.DOCUMENT_CLASS) {
        return;
      }
      /*
      * one node can be child of an Document and a Object Class
      * so the parents can have 2 different types
      */


      if (parentId === 'http://terminusdb.com/schema/system#Document') {
        _rootIndexObj[classId]['type'] = 'Document';
        /*
        * Temporarily, I add the node as a child of Document Group
        */

        _rootIndexObj.DocumentClasses.children.push(_rootIndexObj[classId]);
        /*
        * if the call is child of document it is a document too so I don't need to check more
        */


        return;
      } else if (_rootIndexObj[parentId] && _rootIndexObj[parentId]['type']) {
        /*
        * I have already add the parent Obj and I know the type of the parent
        * I can use the parent type 
        */
        _rootIndexObj[classId]['type'] = _rootIndexObj[parentId].type;
      }
    });
  } else {
    /*
    * if no parents it is  a first level Object class
    */
    _rootIndexObj[classId]['type'] = _elementsName.CLASS_TYPE_NAME.OBJECT_CLASS;

    _rootIndexObj[_elementsName.CLASS_TYPE_NAME.OBJECT_CLASSES].children.push(_rootIndexObj[classId]);
  }
};

var isEnumClassType = function isEnumClassType(classElement, item, _rootIndexObj) {
  /*
  * check if it is a enum node
  */
  if (item["Choices"] !== "system:unknown" && Array.isArray(item["Choices"])) {
    var choicesList = [];
    item["Choices"].forEach(function (choice) {
      var choiceId = getId(choice[0]);
      var choiceLabel = choice[1]['@value'];
      var choiceComment = choice[2]['@value'];
      choicesList.push({
        id: choiceId,
        label: choiceLabel,
        comment: choiceComment
      });
    });
    classElement['type'] = _elementsName.CLASS_TYPE_NAME.CHOICE_CLASS;
    classElement['choices'] = choicesList;

    _rootIndexObj[_elementsName.CLASS_TYPE_NAME.CHOICE_CLASSES].children.push(classElement);

    return true;
  }

  return false;
};

var addElements = function addElements(_rootIndexObj) {
  var dataProvider = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  var bindings = dataProvider.bindings || [];
  bindings.forEach(function (item) {
    var classId = item['Class ID'];
    var description = item['Description']['@value'];

    var _abstract = getAbstractValue(item); //

    /* this is an Object 
    * "Parents": {"@type":"http://www.w3.org/2001/XMLSchema#string", "@value":""}
    */


    if (!_rootIndexObj[classId]) {
      _rootIndexObj[classId] = (0, _modelTreeUtils.getNewNodeTemplate)(classId);
    }
    /*
    * we review this
    */


    var label = getLabel(item, _rootIndexObj[classId]['id']);
    _rootIndexObj[classId]['abstract'] = _abstract;
    _rootIndexObj[classId]['label'] = label;
    _rootIndexObj[classId]['comment'] = description;
    getClassType(classId, item, _rootIndexObj); //add for children

    if (Array.isArray(item['Children'])) {
      /*
      * I can't remove this node before move or remove all the children
      */
      item['Children'].forEach(function (childId) {
        if (!_rootIndexObj[childId]) {
          _rootIndexObj[childId] = (0, _modelTreeUtils.getNewNodeTemplate)(childId, _rootIndexObj[classId].type);
        } else {
          /*
          * if exist I remove it from document group
          */
          if (_rootIndexObj[childId].type === _elementsName.CLASS_TYPE_NAME.DOCUMENT_CLASS && _rootIndexObj[classId].type === _elementsName.CLASS_TYPE_NAME.DOCUMENT_CLASS) {
            (0, _modelTreeUtils.removeElementToArr)(_rootIndexObj[_elementsName.CLASS_TYPE_NAME.DOCUMENT_CLASSES].children, childId);
          }
          /*
          * if it has multi parents type the node is always a document type
          * if child_type id undefined or object type
          */


          if (_rootIndexObj[classId].type && _rootIndexObj[childId].type !== _elementsName.CLASS_TYPE_NAME.DOCUMENT_CLASS && _rootIndexObj[childId].type !== _elementsName.CLASS_TYPE_NAME.CHOICE_CLASS) {
            _rootIndexObj[childId]['type'] = _rootIndexObj[classId].type;
            /*
            * check the other levels of relationship  
            */

            checkChildrenType(_rootIndexObj[childId].children, _rootIndexObj[classId].type);
          }
        } //children property is for d3 hierarchy I add the node only at the first parent 


        if (_rootIndexObj[childId]['parents'].length === 0) {
          _rootIndexObj[classId]['children'].push(_rootIndexObj[childId]);
        } // add the current node as parent				


        _rootIndexObj[childId]['parents'].push(classId); // add child to the current node


        _rootIndexObj[classId]['allChildren'].push(_rootIndexObj[childId]);
      });
    }
  });
};

function checkChildrenType(childrenElements, parentType) {
  childrenElements.forEach(function (childrenEl) {
    childrenEl.type = parentType;
    checkChildrenType(childrenEl.children, parentType);
  });
}
/*
*the list of available parents depends on the type of node
*the list can not have the node currents parents or the currents children
*Object Class type can inherit only from Ordinary Classes 
*Enum Class Type can inherit only from Ordinary Classes 
*Document Class type can inherit from Ordinary Classes and Entity Classes
*/


var availableParentsList = function availableParentsList(classObj, objectTypeList, documentTypeList, _rootIndexObj) {
  var resultListObject = {};
  resultListObject['documentClassArr'] = [];
  resultListObject['objectClassArr'] = removeRelatedElements(classObj, objectTypeList, _rootIndexObj);
  if (classObj.type === _elementsName.CLASS_TYPE_NAME.DOCUMENT_CLASS) resultListObject['documentClassArr'] = removeRelatedElements(classObj, documentTypeList, _rootIndexObj);
  return resultListObject;
};

exports.availableParentsList = availableParentsList;

var removeRelatedElements = function removeRelatedElements(classObj, classesMap, _rootIndexObj) {
  if (classesMap.length === 0) return [];
  var objectClassMap = classesMap.slice();
  (0, _modelTreeUtils.removeElementToArr)(objectClassMap, classObj.name);

  if (classObj.children && classObj.children.length > 0) {
    removeRelatedChildren(classObj.children, objectClassMap);
  }

  if (classObj.parents && classObj.parents.length > 0) removeRelatedParent(classObj.parents, objectClassMap, _rootIndexObj);
  var classList = [];
  objectClassMap.forEach(function (elementName) {
    var elementObj = _rootIndexObj[elementName];
    var label = elementObj.label || elementObj.id;
    classList.push({
      name: elementName,
      value: elementName,
      label: label
    });
  });
  return classList; //[...objectClassMap.values()]
};
/*
* recursive remove related parents
* parents=['parentName001','parentName002'...]
*/


var removeRelatedParent = function removeRelatedParent(parentsList, classesMap, _rootIndexObj) {
  parentsList.forEach(function (parentName, key) {
    /*if(classesMap.has(parentName)){
    	classesMap.delete(parentName)
    }*/
    (0, _modelTreeUtils.removeElementToArr)(classesMap, parentName);
    var parentObj = _rootIndexObj[parentName];
    removeRelatedParent(parentObj.parents, classesMap, _rootIndexObj);
  });
};
/*
* recursive remove related children
*/


var removeRelatedChildren = function removeRelatedChildren(childrenList, classList) {
  childrenList.forEach(function (childObj, key) {
    /*if(classList.has(childObj.name)){
    	classList.delete(childObj.name)
    }*/
    (0, _modelTreeUtils.removeElementToArr)(classList, childObj.name);
    removeRelatedChildren(childObj.children, classList);
  });
};