"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.NodeTree = void 0;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _NodeMenu = require("./NodeMenu");

var _NodeConstants = require("./NodeConstants");

var _elementsName = require("../utils/elementsName");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var NodeTree = function NodeTree(props) {
  var onClick = function onClick(evt) {
    if (props.nodeClick) {
      props.nodeClick(evt.currentTarget.id);
    }
  };

  var width = 100;

  var getLabelWrap = function getLabelWrap(label) {
    if (label.length > 10) {
      return label.substr(0, 9) + '....';
    }

    return label;
  };

  var formatLabel = function formatLabel(label) {
    var minLabel = label.trim().replace(/A | a | an /g, ' ');
    var labelArr = minLabel.trim().split(" ");
    if (labelArr.length === 1) return getLabelWrap(label);
    var distance = 8;
    var startDistance = -(distance * (labelArr.length - 1));
    var labelObj = labelArr.map(function (labelWord, index) {
      return /*#__PURE__*/_react["default"].createElement("tspan", {
        key: "tspan ".concat(index),
        x: 0,
        dy: index === 0 ? startDistance : distance * 2
      }, getLabelWrap(labelWord));
    });
    return labelObj;
  };

  var node = props.node;
  var label = node.data.label || node.data.id || '';
  var comment = node.data.comment || label;

  var getNode = function getNode() {
    var elemStyleType = node.data.type === 'Group' ? node.data.name : node.data.type;
    var elemStyle = _NodeConstants.elementsStyle[elemStyleType] || {};
    var fillColor = elemStyle.fillColor || '#1eadfb';
    var lineSize = elemStyle.lineSize || 2;
    var lineColor = elemStyle.lineColor || '#1eadfb';

    if (node.data["abstract"] === true) {
      fillColor = elemStyle.fillColor_ab;
    }

    if (props.isSelected) {
      lineSize = 4;
      lineColor = '#696969';
    }

    switch (node.data.type) {
      case _elementsName.CLASS_TYPE_NAME.DOCUMENT_CLASS:
        return /*#__PURE__*/_react["default"].createElement("rect", {
          x: -width / 2,
          y: -width / 2,
          width: width,
          height: width,
          fill: fillColor,
          stroke: lineColor,
          strokeWidth: lineSize,
          cursor: 'pointer',
          onClick: props.onClick,
          rx: "15"
        }, /*#__PURE__*/_react["default"].createElement("title", null, comment));

      case _elementsName.CLASS_TYPE_NAME.OBJECT_CLASS:
        return /*#__PURE__*/_react["default"].createElement("circle", {
          markerEnd: props.isSelected ? 'url(#nodeTreeList)' : '',
          r: width / 2 + 3,
          fill: fillColor,
          stroke: lineColor,
          strokeWidth: lineSize,
          cursor: 'pointer',
          onClick: props.onClick
        }, /*#__PURE__*/_react["default"].createElement("title", null, comment));

      case _elementsName.CLASS_TYPE_NAME.CHOICE_CLASS:
        return /*#__PURE__*/_react["default"].createElement("polygon", {
          transform: "rotate(90)",
          points: "0,-60  52,-30  52,30 0,60 -52, 30 -52,-30",
          fill: fillColor,
          stroke: lineColor,
          strokeWidth: lineSize,
          cursor: 'pointer',
          onClick: props.onClick
        }, /*#__PURE__*/_react["default"].createElement("title", null, comment));

      default:
        return /*#__PURE__*/_react["default"].createElement("ellipse", {
          rx: "100",
          ry: "50",
          fill: fillColor,
          stroke: lineColor,
          strokeWidth: lineSize,
          onClick: props.onClick
        });
    }
  };
  /*
    <circle 
  			          markerEnd={props.isSelected ? 'url(#nodeTreeList)' : '' }
  			          r={width/2}		         
  			          fill={fillColor}
  			          stroke={lineColor}
  			          strokeWidth={lineSize}
  			          cursor={'pointer'}
  			      	  onClick={props.onClick}	>	          
  		             <title>
  		        	{comment}
  			        </title>				         
  			        </circle>*/


  var nodex = props.nodex;
  var nodey = props.nodey;
  var menuData = _NodeConstants.groupMenuList[node.data.name] ? _NodeConstants.groupMenuList[node.data.name] : _NodeConstants.nodeMenuList;
  var isEditMode = props.isEditMode === true ? true : false;
  return /*#__PURE__*/_react["default"].createElement("g", {
    indexpos: props.indexPos,
    onClick: onClick,
    className: 'node',
    transform: "translate(".concat(nodex, ",").concat(nodey, ")"),
    name: node.data.name,
    id: node.data.name
  }, getNode(), /*#__PURE__*/_react["default"].createElement("text", {
    fontSize: 14,
    fontFamily: "Arial",
    textAnchor: "middle",
    style: {
      pointerEvents: "none"
    },
    fill: "#000000",
    dominantBaseline: "central",
    alignmentBaseline: "central"
  }, formatLabel(label)), props.isEditMode !== false && props.isSelected && node.data.type !== 'ChoiceClass' && /*#__PURE__*/_react["default"].createElement(_NodeMenu.NodeMenu, {
    setNodeAction: props.setNodeAction,
    width: width,
    nodeId: node.data.name,
    menuList: menuData
  })); //}
};

exports.NodeTree = NodeTree;
NodeTree.propTypes = {
  isEditMode: _propTypes["default"].bool
};
NodeTree.defaultProps = {
  isEditMode: true
};