"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.graphUpdateObject = void 0;

var _actionType = require("./actionType");

var _terminusdbClient = _interopRequireWildcard(require("@terminusdb/terminusdb-client"));

var _elementsName = require("../utils/elementsName");

var _modelTreeUtils = require("../utils/modelTreeUtils");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

var graphUpdateObject = function graphUpdateObject() {
  var newNodesList = new Map();
  var newPropertiesList = new Map();
  var deleteNodesList = new Map();
  var deletePropertiesList = new Map();
  var updateChoiceList = new Map();
  var updateTriple = new Map();
  /*
  * for the interface I can change the parentship from child to parent
  */

  var changeParentList = new Map();

  var addParent = function addParent(currentNode, newNode, isChoiceClass) {
    if (currentNode.type === _elementsName.CLASS_TYPE_NAME.SCHEMA_GROUP) {
      if (currentNode.name === _elementsName.CLASS_TYPE_NAME.DOCUMENT_CLASSES) {
        newNode.parent = ['Document'];
        newNode.parents = [];
        newNode.type = _elementsName.CLASS_TYPE_NAME.DOCUMENT_CLASS;
      } else {
        newNode.parent = [];
        newNode.parents = [];
        var nodeType = _elementsName.CLASS_TYPE_NAME.OBJECT_CLASS;

        if (isChoiceClass) {
          nodeType = _elementsName.CLASS_TYPE_NAME.CHOICE_CLASS;
          newNode.choices = [];
        }

        newNode.type = nodeType;
      }
    } else {
      newNode.parent = [currentNode.id];
      newNode.parents = [currentNode.name];
      newNode.type = currentNode.type;
    }
  };

  var updateChoicesList = function updateChoicesList(choiceClass) {
    if (choiceClass.newElement === true) return;
    updateChoiceList.set(choiceClass.name, choiceClass);
  };
  /*
  * name is internal unique reference 
  */


  var updateTripleElement = function updateTripleElement(propName, propValue, currentElement) {
    /*
    * I don't need to add a triple change because this is a new node
    */
    if (currentElement.newElement === true) return;
    var values = {};

    if (updateTriple.has(currentElement.name)) {
      values = updateTriple.get(currentElement.name);
      values[propName] = propValue;
    } else {
      values[propName] = propValue;
      updateTriple.set(currentElement.name, values);
    }
    /*
    * if the element is a property
    */


    if (currentElement.domain) {
      values['domain'] = currentElement.domain;
    }
  };
  /*
  * newNodeParent : parent of the new node 
  * newNodeChild  : child of the new node
  */


  var addNodeToTree = function addNodeToTree(newNodeParent) {
    var newNodeChild = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
    var isChoiceClass = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
    var elementModel = (0, _modelTreeUtils.getNewNodeTemplate)();
    addParent(newNodeParent, elementModel, isChoiceClass);
    newNodesList.set(elementModel['name'], elementModel);

    if (newNodeChild) {
      /*
      * add the parent relationship to the child node
      */
      changeNodeParent(newNodeChild.name, elementModel['name'], _actionType.ADD_PARENT);
      newNodeChild.parents.push(elementModel['name']);
    }

    return elementModel;
  };

  var changeNodeParent = function changeNodeParent(childName, parentName, action) {
    var parentsList = [];

    if (changeParentList.has(childName)) {
      parentsList = changeParentList.get(childName);
    } else {
      changeParentList.set(childName, parentsList);
    }

    parentsList.push({
      childNode: childName,
      parentNode: parentName,
      action: action
    });
  };

  var removeNode = function removeNode(nodeObj) {
    if (nodeObj.newElement === true) {
      newNodesList["delete"](nodeObj.name);
    } else {
      deleteNodesList.set(nodeObj.name, nodeObj);
    }
  };
  /*
  * add a new property when I'll save 
  * I have to replace the domain if the property is for a new node 
  */


  var addPropertyToClass = function addPropertyToClass(domainNodeName, propertyType, propertyRange) {
    var newName = "PROPERTY_".concat(new Date().getTime());
    var elementModel = {
      name: newName,
      id: "",
      label: "NEW PROPERTY",
      comment: "",
      type: propertyType,
      newElement: true,
      range: propertyRange,
      domain: domainNodeName
    };
    newPropertiesList.set(newName, elementModel);
    return elementModel;
  };

  var removePropertyToClass = function removePropertyToClass(propertyObject) {
    /*
    * if the property is only local (not saved in the db) 
    * I can remove it from the list 
    */
    if (propertyObject.newElement === true) {
      newPropertiesList["delete"](propertyObject.name);
    } else {
      deletePropertiesList.set(propertyObject.name, propertyObject);
    }
  };
  /*
  * In new node we use a internal name for reference 
  */


  var getPropertyObjForSave = function getPropertyObjForSave(propertyObj) {
    var copyNode = JSON.parse(JSON.stringify(propertyObj));
    copyNode.description = copyNode.comment;
    copyNode['domain'] = getRealId(copyNode.domain);

    if (propertyObj.type === _elementsName.PROPERTY_TYPE_NAME.OBJECT_PROPERTY) {
      copyNode['range'] = getRealId(copyNode.range);
    }
    /*
    * add the prefix for save the property
    */


    copyNode.id = checkNewId(propertyObj.id);
    return copyNode;
  };
  /*
  * the new node has not the prefix I have to add the prefix
  */


  var getRealId = function getRealId(elementName) {
    if (newNodesList.has(elementName)) {
      return checkNewId(newNodesList.get(elementName).id);
    }

    return elementName;
  };

  var checkNewId = function checkNewId(id) {
    if (id.indexOf(":") === -1) {
      return "scm:".concat(id);
    }

    return id;
  };

  var formatChoiceListForWoql = function formatChoiceListForWoql(choicelist) {
    var choices = [];
    choicelist.forEach(function (item) {
      var id = checkNewId(item.id);
      var choiceArr = [id, item.label, item.comment];
      choices.push(choiceArr);
    });
    return choices;
  };

  var savedObjectToWOQL = function savedObjectToWOQL(_rootIndexObj, _propertiesList) {
    var WOQL = _terminusdbClient["default"].WOQL;
    var andValues = [];
    newNodesList.forEach(function (node, key) {
      /*if(node.id===undefined){
      	alert(`The node ${node.name}`)
      }*/
      if (node.type !== _elementsName.CLASS_TYPE_NAME.CHOICE_CLASS) {
        var newNode = {
          id: node.id,
          label: node.label,
          description: node.comment,
          parent: node.parent,
          "abstract": node["abstract"]
        };
        andValues.push(WOQL.insert_class_data(newNode));
      } else {
        var choices = formatChoiceListForWoql(node.choices);
        andValues.push(WOQL.generateChoiceList(node.id, node.label, node.comment, choices));
      }
    });
    newPropertiesList.forEach(function (property, key) {
      var propertyObj = getPropertyObjForSave(property);
      andValues.push(WOQL.insert_property_data(propertyObj));
    });
    changeParentList.forEach(function (parentActArr, key) {
      parentActArr.forEach(function (item, key) {
        var idChild = getRealId(item.childNode);
        var idParent = getRealId(item.parentNode);

        switch (item.action) {
          case _actionType.ADD_PARENT:
            andValues.push(WOQL.add_quad(idChild, "subClassOf", idParent, "schema/main"));
            break;

          case _actionType.REMOVE_PARENT:
            andValues.push(WOQL.delete_quad(idChild, "subClassOf", idParent, "schema/main"));
        }
      });
    });
    updateTriple.forEach(function (valuesObject, subject) {
      /*
      * update triple only if the object exists
      */
      if (_rootIndexObj[subject] || _propertiesList.get(subject)) {
        var subjectId = getRealId(subject);
        var addCard = true;

        for (var vname in valuesObject) {
          if (vname === "domain") continue;
          /*
          * I add the cardinality value all together
          */

          if (['min', 'max', 'cardinality'].indexOf(vname) > -1 && addCard) {
            addCard = false;

            var propertyObj = _propertiesList.get(subject);

            andValues.push(updateCardinality(WOQL, subjectId, propertyObj));
          } else {
            if (vname === 'abstract') {
              andValues.push(updateAbstract(WOQL, subjectId, valuesObject[vname]));
            } else {
              andValues.push(WOQL.update_quad(subjectId, vname, valuesObject[vname], 'schema/main'));
            }
          }
        }
      }
    });
    deleteNodesList.forEach(function (nodeObj) {
      if (nodeObj.type === _elementsName.CLASS_TYPE_NAME.CHOICE_CLASS) {
        andValues.push(WOQL.opt().deleteChoiceList(nodeObj.id));
      } else {
        var varName = "v:".concat(nodeObj.id);
        andValues.push(WOQL.delete_class(nodeObj.name, null, varName));
      }
    });
    deletePropertiesList.forEach(function (proObj) {
      var varName = "v:".concat(proObj.id);
      andValues.push(WOQL.delete_property(proObj.name, null, varName));
    });
    /*
    * if I change something in the choicelist I have to remove and add again
    * I do this only if the element exists
    */

    updateChoiceList.forEach(function (choiceObj, choiceName) {
      if (_rootIndexObj[choiceName]) {
        var choiceId = getRealId(choiceName);
        var choices = formatChoiceListForWoql(choiceObj.choices);
        andValues.push(WOQL.updateChoiceList(choiceId, choiceObj.label, choiceObj.comment, choices));
      }
    });
    /*
    * I have to review the save mode
    */

    if (andValues.length === 0) return undefined;
    var query = WOQL.and.apply(WOQL, andValues);
    return query;
  };

  var updateAbstract = function updateAbstract(WOQL, subjectId, value) {
    if (value === true) {
      return WOQL.update_quad(subjectId, 'system:tag', 'system:abstract', 'schema/main');
    }

    return WOQL.opt(WOQL.quad(subjectId, 'system:tag', "system:abstract", "schema/main").delete_quad(subjectId, 'system:tag', "system:abstract", "schema/main"));
  };

  var addCardinality = function addCardinality(WOQL, subjectId, cardName, cardType, value, domainId) {
    return WOQL.add_quad(cardName, "type", "owl:Restriction", "schema/main").add_quad(cardName, "owl:onProperty", subjectId, "schema/main").add_quad(cardName, cardType, WOQL.literal(value, "xsd:nonNegativeInteger"), "schema/main").add_quad(domainId, "subClassOf", cardName, "schema/main");
  };

  var deleteSubClass = function deleteSubClass(WOQL, domainId, cardName, varName) {
    return WOQL.opt(WOQL.quad(domainId, "rdfs:subClassOf", varName, "schema/main").eq(varName, "".concat(cardName)).delete_quad(domainId, "rdfs:subClassOf", varName, "schema/main"));
  };

  var updateCardinality = function updateCardinality(WOQL, subjectId, propertyObj) {
    var domainId = getRealId(propertyObj.domain);
    /*
    *remove alle the cardinality restriction
    */

    var query = [WOQL.opt(WOQL.quad("v:Restriction", "owl:onProperty", subjectId, "schema/main").delete_quad("v:Restriction", "owl:onProperty", subjectId, "schema/main"))];

    if (propertyObj.min_start) {
      var minStartName = "".concat(subjectId, "_min_").concat(propertyObj.min_start);
      query.push(deleteSubClass(WOQL, domainId, minStartName, 'v:minStart'));
    }

    if (propertyObj.max_start) {
      var maxStartName = "".concat(subjectId, "_max_").concat(propertyObj.max_start);
      query.push(deleteSubClass(WOQL, domainId, maxStartName, 'v:maxStart'));
    }

    if (propertyObj.cardinality_start) {
      var cardStartName = "".concat(subjectId, "_cardinality_").concat(propertyObj.cardinality_start);
      query.push(deleteSubClass(WOQL, domainId, cardStartName, 'v:carStart'));
    }

    if (propertyObj.min && propertyObj.max && propertyObj.min === propertyObj.max) {
      var cardName = subjectId + '_card_' + propertyObj.min;
      var value = propertyObj.min;
      query.push(addCardinality(WOQL, subjectId, cardName, "owl:cardinality", value, domainId));
    } else {
      if (propertyObj.min && parseFloat(propertyObj.min) > 0) {
        var cardNameMin = subjectId + '_min_' + propertyObj.min;
        var minValue = propertyObj.min;
        query.push(addCardinality(WOQL, subjectId, cardNameMin, "owl:minCardinality", minValue, domainId));
      }

      if (propertyObj.max && parseFloat(propertyObj.max) > 0) {
        var cardNameMax = subjectId + '_max_' + propertyObj.max;
        var maxValue = propertyObj.max;
        query.push(addCardinality(WOQL, subjectId, cardNameMax, "owl:maxCardinality", maxValue, domainId));
      }
    }

    var tmp = WOQL.and.apply(WOQL, query); //console.log("__QUERY___",tmp.prettyPrint())

    return tmp;
  };

  return {
    updateChoicesList: updateChoicesList,
    addNodeToTree: addNodeToTree,
    changeNodeParent: changeNodeParent,
    addPropertyToClass: addPropertyToClass,
    removePropertyToClass: removePropertyToClass,
    savedObjectToWOQL: savedObjectToWOQL,
    removeNode: removeNode,
    updateTripleElement: updateTripleElement
  };
}; //new node 
//new property

/*

*/
//update triple (value to a prop)
//cardinality (change chardinality value)
//if they change locally like add new node remove new node 
//if they remove or change some


exports.graphUpdateObject = graphUpdateObject;